__author__ = "Jia Jia"
__author_email__ = "angersax@sina.com"
__description__ = "SCANOSS Report"
__license__ = "Apache-2.0"
__title__ = "ossreport"
__url__ = "https://github.com/craftslab/ossreport"
__version__ = "1.1.0"
